@@ -49,7 +49,6 @@ import org.apache.sshd.client.session.ClientSession;
 import org.apache.sshd.common.Factory;
 import org.apache.sshd.common.channel.Channel;
 import org.apache.sshd.common.file.virtualfs.VirtualFileSystemFactory;
-import org.apache.sshd.common.io.BuiltinIoServiceFactoryFactories;
 import org.apache.sshd.common.random.Random;
 import org.apache.sshd.common.session.Session;
 import org.apache.sshd.common.util.GenericUtils;
@@ -99,9 +98,6 @@ public class ScpTest extends AbstractScpTestSupport {
 
     @Test
     public void testNormalizedScpRemotePaths() throws Exception {
-        // see SSHD-822
-        assumeNotIoServiceProvider(
-                EnumSet.of(BuiltinIoServiceFactoryFactories.NETTY, BuiltinIoServiceFactoryFactories.MINA));
         Path targetPath = detectTargetFolder();
         Path parentPath = targetPath.getParent();
         Path scpRoot = CommonTestSupportUtils.resolve(targetPath,
@@ -275,9 +271,6 @@ public class ScpTest extends AbstractScpTestSupport {
 
     @Test
     public void testScpNativeOnSingleFile() throws Exception {
-        // see SSHD-822
-        assumeNotIoServiceProvider(EnumSet.of(BuiltinIoServiceFactoryFactories.NETTY));
-
         String data = getClass().getName() + "#" + getCurrentTestName() + IoUtils.EOL;
 
         Path targetPath = detectTargetFolder();
@@ -327,9 +320,6 @@ public class ScpTest extends AbstractScpTestSupport {
 
     @Test
     public void testScpNativeOnMultipleFiles() throws Exception {
-        // see SSHD-822
-        assumeNotIoServiceProvider(EnumSet.of(BuiltinIoServiceFactoryFactories.MINA, BuiltinIoServiceFactoryFactories.NETTY));
-
         try (CloseableScpClient scp = createCloseableScpClient()) {
             Path targetPath = detectTargetFolder();
             Path parentPath = targetPath.getParent();
@@ -833,6 +823,29 @@ public class ScpTest extends AbstractScpTestSupport {
         }
     }
 
+    @Test // see GH-428
+    public void testMissingRemoteFile() throws Exception {
+        Path targetPath = detectTargetFolder();
+        Path scpRoot = CommonTestSupportUtils.resolve(targetPath,
+                ScpHelper.SCP_COMMAND_PREFIX, getClass().getSimpleName(), getCurrentTestName());
+        Path localDir = assertHierarchyTargetFolderExists(scpRoot.resolve("local"));
+        Path remoteDir = assertHierarchyTargetFolderExists(scpRoot.resolve("remote"));
+        Path missingLocal = localDir.resolve("missing.txt");
+        Path missingRemote = remoteDir.resolve(missingLocal.getFileName());
+        Files.deleteIfExists(missingLocal);
+        Files.deleteIfExists(missingRemote);
+        assertFalse("Remote file not deleted", Files.exists(missingRemote));
+
+        String remotePath = CommonTestSupportUtils.resolveRelativeRemotePath(targetPath.getParent(), missingRemote);
+        try (CloseableScpClient scp = createCloseableScpClient()) {
+            scp.download(remotePath, missingLocal.toString());
+            fail("Unexpected success to copy non-existent remote file");
+        } catch (ScpException e) {
+            assertEquals("Mismatched SCP failure code", ScpAckInfo.ERROR, e.getExitStatus().intValue());
+        }
+
+    }
+
     @Test
     public void testJschScp() throws Exception {
         com.jcraft.jsch.Session session = getJschSession();
@@ -938,11 +951,11 @@ public class ScpTest extends AbstractScpTestSupport {
     protected String readFile(com.jcraft.jsch.Session session, String path, Path target) throws Exception {
         ChannelExec c = (ChannelExec) session.openChannel(Channel.CHANNEL_EXEC);
         c.setCommand("scp -f " + path);
-        c.connect();
 
         String fileName = Objects.toString(target.getFileName(), null);
         try (OutputStream os = c.getOutputStream();
              InputStream is = c.getInputStream()) {
+            c.connect();
 
             os.write(0);
             os.flush();
@@ -975,10 +988,10 @@ public class ScpTest extends AbstractScpTestSupport {
     protected String readDir(com.jcraft.jsch.Session session, String path, Path target) throws Exception {
         ChannelExec c = (ChannelExec) session.openChannel(Channel.CHANNEL_EXEC);
         c.setCommand("scp -r -f " + path);
-        c.connect();
 
         try (OutputStream os = c.getOutputStream();
              InputStream is = c.getInputStream()) {
+            c.connect();
             ScpAckInfo.sendOk(os, StandardCharsets.UTF_8);
 
             String header = ScpIoUtils.readLine(is, StandardCharsets.UTF_8, false);
@@ -1017,10 +1030,10 @@ public class ScpTest extends AbstractScpTestSupport {
         ChannelExec c = (ChannelExec) session.openChannel(Channel.CHANNEL_EXEC);
         String command = "scp -f " + path;
         c.setCommand(command);
-        c.connect();
 
         try (OutputStream os = c.getOutputStream();
              InputStream is = c.getInputStream()) {
+            c.connect();
 
             ScpAckInfo.sendOk(os, StandardCharsets.UTF_8);
             assertEquals("Mismatched response for command: " + command, ScpAckInfo.ERROR, is.read());
@@ -1033,10 +1046,10 @@ public class ScpTest extends AbstractScpTestSupport {
         ChannelExec c = (ChannelExec) session.openChannel(Channel.CHANNEL_EXEC);
         String command = "scp -t " + path;
         c.setCommand(command);
-        c.connect();
 
         try (OutputStream os = c.getOutputStream();
              InputStream is = c.getInputStream()) {
+            c.connect();
 
             assertAckReceived(is, command);
 
@@ -1072,10 +1085,10 @@ public class ScpTest extends AbstractScpTestSupport {
         ChannelExec c = (ChannelExec) session.openChannel(Channel.CHANNEL_EXEC);
         String command = "scp -t " + path;
         c.setCommand(command);
-        c.connect();
 
         try (OutputStream os = c.getOutputStream();
              InputStream is = c.getInputStream()) {
+            c.connect();
 
             assertAckReceived(is, command);
 
@@ -1092,10 +1105,10 @@ public class ScpTest extends AbstractScpTestSupport {
         ChannelExec c = (ChannelExec) session.openChannel(Channel.CHANNEL_EXEC);
         String command = "scp -t -r " + path;
         c.setCommand(command);
-        c.connect();
 
         try (OutputStream os = c.getOutputStream();
              InputStream is = c.getInputStream()) {
+            c.connect();
 
             assertAckReceived(is, command);
             assertAckReceived(os, is, "D0755 0 " + dirName);
